import os
import sys

import click

from .util import honcho_parse_env


ENV_DEFAULT = '.env'


@click.command(
    help='Imports a module passed on the commandline, instantiates an app by '
         'calling imported_module.create_app() with an optional configuration '
         'file and runs it in debug mode.'
)
@click.argument('module_name')
@click.option('--configfile', '-c',
              type=click.Path(exists=True, dir_okay=False),
              help='Configuration file to pass as the first parameter to '
                   'create_app')
@click.option('--debug/--no-debug', '-d/-D', default=True,
              help='Enabled/disable debug (enabled by default)')
@click.option('--hostname', '-H', default='localhost',
              help='Hostname to bind to. Defaults to localhost')
@click.option('--port', '-p', type=int, default=5000,
              help='Port to listen on. Defaults to 5000')
@click.option('--ssl', '-S', flag_value='adhoc', default=None,
              help='Enable SSL with a self-signed cert')
@click.option('--env', '-e', default=None,
              type=click.Path(exists=True, dir_okay=False),
              help='Load environment variables from file (default: ".env")')
@click.option('--flask-debug/--no-flask-debug', '-e/-E', default=None,
              help='Enable/disable Flask-Debug or Flask-DebugToolbar '
                   'extensions (default: same as --debug)')
def main(module_name, configfile, debug, hostname, port, ssl, env,
         flask_debug):
    # extra files for the reloader to watch
    extra_files = []

    if configfile:
        extra_files.append(configfile)

    try:
        import importlib
    except ImportError:
        click.echo('You do not have importlib installed. Please install a '
                   'backport for versions < 2.7/3.1 of it first.')
        sys.exit(1)

    msgs = []

    if flask_debug is None:
        flask_debug = debug

    Debug = None

    if flask_debug:
        try:
            from flask_debug import Debug
        except ImportError:
            Debug = None

        try:
            from flask_debugtoolbar import DebugToolbarExtension
        except ImportError:
            DebugToolbarExtension = None

    if env is None and os.path.exists(ENV_DEFAULT):
        env = ENV_DEFAULT

    if env:
        extra_files.append(env)
        buf = open(env).read()
        os.environ.update(honcho_parse_env(buf))

    mod = importlib.import_module(module_name)
    app = mod.create_app(configfile)

    if Debug:
        Debug(app)
        app.config['SERVER_NAME'] = '{}:{}'.format(hostname, port)

        # taking off the safety wheels
        app.config['FLASK_DEBUG_DISABLE_STRICT'] = True

    if DebugToolbarExtension:
        # Flask-Debugtoolbar does not check for debugging settings at runtime.
        # this hack enabled debugging if desired before initializing the
        # extension
        if debug:
            app.debug = True

            # set the SECRET_KEY, but only if we're in debug-mode
            if not app.config.get('SECRET_KEY', None):
                msgs.append('SECRET_KEY not set, using insecure "devkey"')
                app.config['SECRET_KEY'] = 'devkey'

        DebugToolbarExtension(app)

    def on_off(ext):
        return 'on' if ext is not None else 'off'

    msgs.insert(0, 'Flask-Debug: {}'.format(on_off(Debug)))
    msgs.insert(0, 'Flask-DebugToolbar: {}'.format(
        on_off(DebugToolbarExtension))
    )

    if msgs:
        click.echo(' * {}'.format(', '.join(msgs)))

    app.run(hostname, port, ssl_context=ssl, debug=debug,
            extra_files=extra_files)



# the function below has been lifted from the honcho source
#
# Copyright (c) 2012 Nick Stenning, http://whiteink.com/
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
import re


def honcho_parse_env(content):
        values = {}
        for line in content.splitlines():
            m1 = re.match(r'\A([A-Za-z_0-9]+)=(.*)\Z', line)
            if m1:
                key, val = m1.group(1), m1.group(2)

                m2 = re.match(r"\A'(.*)'\Z", val)
                if m2:
                    val = m2.group(1)

                m3 = re.match(r'\A"(.*)"\Z', val)
                if m3:
                    val = re.sub(r'\\(.)', r'\1', m3.group(1))

                values[key] = val
        return values
# end of honcho source
